==> server-api-impl-22.2.0-802.1338-sources.jar: com/xebialabs/deployit/core/service/status/DeploymentPackageStatusService.scala <==
package com.xebialabs.deployit.core.service.status

import ai.digital.deploy.webhook.WebhookServer
import com.xebialabs.deployit.ServerConfiguration
import com.xebialabs.deployit.engine.api.dto.{ApplicationDeploymentPackage, ApplicationDeploymentPackageState, DeployedApplicationPackage}
import com.xebialabs.deployit.engine.spi.exception.DeployitException
import com.xebialabs.deployit.repository.RepositoryService
import com.xebialabs.deployit.service.dependency.RepositoryServiceAware
import com.xebialabs.deployit.util.PasswordEncrypter
import grizzled.slf4j.Logging
import org.apache.http.client.config.RequestConfig
import org.apache.http.conn.ssl.NoopHostnameVerifier
import org.apache.http.impl.client.{HttpClients, SystemDefaultCredentialsProvider}
import org.apache.http.impl.conn.SystemDefaultRoutePlanner
import org.apache.http.ssl.SSLContextBuilder
import org.springframework.beans.factory.annotation.{Autowired, Qualifier}
import org.springframework.boot.web.client.RestTemplateBuilder
import org.springframework.context.annotation.{Bean, Configuration}
import org.springframework.http.{HttpEntity, HttpHeaders, HttpMethod}
import org.springframework.http.HttpStatus.Series
import org.springframework.http.client.{ClientHttpResponse, HttpComponentsClientHttpRequestFactory}
import org.springframework.stereotype.Service
import org.springframework.util.LinkedMultiValueMap
import org.springframework.web.client.{ResponseErrorHandler, RestTemplate}

import java.io.File
import java.net.ProxySelector
import java.security.cert.X509Certificate
import java.time.Duration
import java.util.concurrent.CompletableFuture
import scala.jdk.CollectionConverters._

trait DeploymentPackageStatusService {
  def changeApplicationDeploymentPackageState(state: ApplicationDeploymentPackageState): Unit

  def createApplicationDeploymentPackage(applicationDeploymentPackage: ApplicationDeploymentPackage): Unit

  def deleteApplicationDeploymentPackage(applicationDeploymentPackage: ApplicationDeploymentPackage): Unit

  def createDeployedApplicationPackage(deployedApplicationPackage: DeployedApplicationPackage): Unit

  def deleteDeployedApplicationPackage(deployedApplicationPackage: DeployedApplicationPackage): Unit
}

@Configuration
class WebhookDeploymentPackageStatusConfiguration(serverConfiguration: ServerConfiguration) {
  def webhookRestTemplateResponseErrorHandler(): ResponseErrorHandler =
    new ResponseErrorHandler() {
      override def hasError(httpResponse: ClientHttpResponse): Boolean =
        httpResponse.getStatusCode.series() == Series.CLIENT_ERROR ||
          httpResponse.getStatusCode.series() == Series.SERVER_ERROR

      override def handleError(response: ClientHttpResponse): Unit =
        response.getStatusCode.series() match {
          case Series.SERVER_ERROR =>
            throw DeploymentPackageStatusServerException(response.getStatusCode.getReasonPhrase)
          case Series.CLIENT_ERROR =>
            throw DeploymentPackageStatusClientException(response.getStatusCode.getReasonPhrase)
        }
    }

  @Bean
  def webhookServiceRestTemplate(builder: RestTemplateBuilder): RestTemplate = {
    val sslContextBuilder = new SSLContextBuilder
    val httpClientBuilder = HttpClients.custom
    val timeout = Duration.ofSeconds(1).toMillis.toInt
    if (serverConfiguration.isSsl)
      Option(serverConfiguration.getKeyStorePath)
        .map(path =>
          sslContextBuilder.loadTrustMaterial(new File(path),
            Option(ServerConfiguration.getInstance().getKeyStorePassword)
              .map(PasswordEncrypter.getInstance().ensureDecrypted).map(_.toCharArray).orNull
          )
        )
        .getOrElse(sslContextBuilder.loadTrustMaterial(null, (_: Array[X509Certificate], _: String) => true))
    else
      sslContextBuilder.loadTrustMaterial(null, (_: Array[X509Certificate], _: String) => true)
    httpClientBuilder.setSSLHostnameVerifier(new NoopHostnameVerifier)
    httpClientBuilder.setRoutePlanner(new SystemDefaultRoutePlanner(ProxySelector.getDefault))
    httpClientBuilder.setDefaultCredentialsProvider(new SystemDefaultCredentialsProvider)
    httpClientBuilder
      .setSSLContext(sslContextBuilder.build)
      .setDefaultRequestConfig(RequestConfig.custom.setSocketTimeout(timeout).setConnectTimeout(timeout).build)
    builder
      .requestFactory(() => new HttpComponentsClientHttpRequestFactory(httpClientBuilder.build()))
      .errorHandler(webhookRestTemplateResponseErrorHandler())
      .build()
  }
}

@Service
class WebhookDeploymentPackageStatusClient @Autowired()(
                                                         val repositoryService: RepositoryService,
                                                         @Qualifier("webhookServiceRestTemplate") webhookServiceRestTemplate: RestTemplate
                                                       ) extends DeploymentPackageStatusService with RepositoryServiceAware with Logging {
  val contentType = "application/json"

  override def changeApplicationDeploymentPackageState(state: ApplicationDeploymentPackageState): Unit =
    executeRequest(HttpMethod.PUT, state)

  override def createApplicationDeploymentPackage(applicationDeploymentPackage: ApplicationDeploymentPackage): Unit =
    executeRequest(HttpMethod.POST, applicationDeploymentPackage)

  override def deleteApplicationDeploymentPackage(applicationDeploymentPackage: ApplicationDeploymentPackage): Unit =
    executeRequest(HttpMethod.DELETE, applicationDeploymentPackage)

  override def createDeployedApplicationPackage(deployedApplicationPackage: DeployedApplicationPackage): Unit =
    executeRequest(HttpMethod.POST, deployedApplicationPackage)

  override def deleteDeployedApplicationPackage(deployedApplicationPackage: DeployedApplicationPackage): Unit =
    executeRequest(HttpMethod.DELETE, deployedApplicationPackage)

  private def getServers: Seq[WebhookServer] =
    repositoryService.
      listEntities[WebhookServer](typedSearchParameters[WebhookServer]).asScala.filter(_.enabled).toVector

  private def executeRequest[T](method: HttpMethod, requestBody: T): Unit =
    getServers.foreach(server => sendAsync(server, method, requestBody))

  private def sendAsync[T](server: WebhookServer, method: HttpMethod, requestBody: T) = {
    CompletableFuture.runAsync(() => {
      val headers = new LinkedMultiValueMap[String, String]()
      if (!server.noAuthentication) {
        headers.add(server.tokenHeader, server.tokenValue)
        headers.add(HttpHeaders.CONTENT_TYPE, contentType)
      }
      webhookServiceRestTemplate.exchange(
        server.webhookServerURL,
        method,
        new HttpEntity[T](requestBody, headers),
        classOf[Void]
      )
    }).whenComplete((_, ex) => {
      Option(ex) match {
        case Some(exception) =>
          logger.warn(s"An error occurred when sending webhook event to ${server.webhookServerURL}: ${exception.getMessage}")
      }
    })
  }
}

final case class DeploymentPackageStatusServerException(message: String) extends DeployitException(message)
final case class DeploymentPackageStatusClientException(message: String) extends DeployitException(message)

==> server-api-impl-22.3.0-sources.jar: com/xebialabs/deployit/core/service/status/DeploymentPackageStatusService.scala <==
package com.xebialabs.deployit.core.service.status

import ai.digital.deploy.task.status.util.DeploymentStatusUtils
import ai.digital.deploy.webhook.WebhookServer
import com.xebialabs.deployit.ServerConfiguration
import com.xebialabs.deployit.engine.api.dto.{ApplicationDeploymentPackage, ApplicationDeploymentPackageState, DeployedApplicationPackage}
import com.xebialabs.deployit.engine.spi.exception.DeployitException
import com.xebialabs.deployit.plugin.api.udm.Application
import com.xebialabs.deployit.repository.RepositoryService
import com.xebialabs.deployit.service.dependency.RepositoryServiceAware
import com.xebialabs.deployit.util.PasswordEncrypter
import grizzled.slf4j.Logging
import org.apache.http.client.config.RequestConfig
import org.apache.http.conn.ssl.NoopHostnameVerifier
import org.apache.http.impl.client.{HttpClients, SystemDefaultCredentialsProvider}
import org.apache.http.impl.conn.SystemDefaultRoutePlanner
import org.apache.http.ssl.SSLContextBuilder
import org.springframework.beans.factory.annotation.{Autowired, Qualifier}
import org.springframework.boot.web.client.RestTemplateBuilder
import org.springframework.context.annotation.{Bean, Configuration}
import org.springframework.http.{HttpEntity, HttpHeaders, HttpMethod}
import org.springframework.http.HttpStatus.Series
import org.springframework.http.client.{ClientHttpResponse, HttpComponentsClientHttpRequestFactory}
import org.springframework.stereotype.Service
import org.springframework.util.LinkedMultiValueMap
import org.springframework.web.client.{ResponseErrorHandler, RestTemplate}

import java.io.File
import java.net.ProxySelector
import java.security.cert.X509Certificate
import java.time.Duration
import java.util.concurrent.CompletableFuture
import scala.jdk.CollectionConverters._

trait DeploymentPackageStatusService {
  def send(state: ApplicationDeploymentPackageState): Unit

  def send(applicationDeploymentPackage: ApplicationDeploymentPackage): Unit

  def send(deployedApplicationPackage: DeployedApplicationPackage): Unit
}

@Configuration
class WebhookDeploymentPackageStatusConfiguration(serverConfiguration: ServerConfiguration) {
  def webhookRestTemplateResponseErrorHandler(): ResponseErrorHandler =
    new ResponseErrorHandler() {
      override def hasError(httpResponse: ClientHttpResponse): Boolean =
        httpResponse.getStatusCode.series() == Series.CLIENT_ERROR ||
          httpResponse.getStatusCode.series() == Series.SERVER_ERROR

      override def handleError(response: ClientHttpResponse): Unit =
        response.getStatusCode.series() match {
          case Series.SERVER_ERROR =>
            throw DeploymentPackageStatusServerException(response.getStatusCode.getReasonPhrase)
          case Series.CLIENT_ERROR =>
            throw DeploymentPackageStatusClientException(response.getStatusCode.getReasonPhrase)
        }
    }

  @Bean
  def webhookServiceRestTemplate(builder: RestTemplateBuilder): RestTemplate = {
    val sslContextBuilder = new SSLContextBuilder
    val httpClientBuilder = HttpClients.custom
    val timeout = Duration.ofSeconds(1).toMillis.toInt
    if (serverConfiguration.isSsl)
      Option(serverConfiguration.getKeyStorePath)
        .map(path =>
          sslContextBuilder.loadTrustMaterial(new File(path),
            Option(ServerConfiguration.getInstance().getKeyStorePassword)
              .map(PasswordEncrypter.getInstance().ensureDecrypted).map(_.toCharArray).orNull
          )
        )
        .getOrElse(sslContextBuilder.loadTrustMaterial(null, (_: Array[X509Certificate], _: String) => true))
    else
      sslContextBuilder.loadTrustMaterial(null, (_: Array[X509Certificate], _: String) => true)
    httpClientBuilder.setSSLHostnameVerifier(new NoopHostnameVerifier)
    httpClientBuilder.setRoutePlanner(new SystemDefaultRoutePlanner(ProxySelector.getDefault))
    httpClientBuilder.setDefaultCredentialsProvider(new SystemDefaultCredentialsProvider)
    httpClientBuilder
      .setSSLContext(sslContextBuilder.build)
      .setDefaultRequestConfig(RequestConfig.custom.setSocketTimeout(timeout).setConnectTimeout(timeout).build)
    builder
      .requestFactory(() => new HttpComponentsClientHttpRequestFactory(httpClientBuilder.build()))
      .errorHandler(webhookRestTemplateResponseErrorHandler())
      .build()
  }
}

@Service
class WebhookDeploymentPackageStatusClient @Autowired()(
                                                         val repositoryService: RepositoryService,
                                                         @Qualifier("webhookServiceRestTemplate") webhookServiceRestTemplate: RestTemplate
                                                       ) extends DeploymentPackageStatusService with RepositoryServiceAware with Logging {
  final val ContentType = "application/json"

  override def send(state: ApplicationDeploymentPackageState): Unit =
    executeRequest(state, state.applicationUid)

  override def send(applicationDeploymentPackage: ApplicationDeploymentPackage): Unit =
    executeRequest(applicationDeploymentPackage, applicationDeploymentPackage.applicationUid)

  override def send(deployedApplicationPackage: DeployedApplicationPackage): Unit =
    executeRequest(deployedApplicationPackage, deployedApplicationPackage.applicationUid)

  private def getServers: Seq[WebhookServer] =
    repositoryService.
      listEntities[WebhookServer](typedSearchParameters[WebhookServer]).asScala.filter(_.enabled).toVector

  private def executeRequest[T](requestBody: T, applicationUid: String): Unit = {
    val applicationDir = DeploymentStatusUtils.removeSuffixFromPath(repositoryService.readByReferenceId[Application](applicationUid).getId)
    getServers.foreach(server => if (server.filteredFolders.asScala.toList.isEmpty || server.filteredFolders.asScala.toList.exists(folder => applicationDir.contains(folder))) sendAsync(server, requestBody))
  }

  private def sendAsync[T](server: WebhookServer, requestBody: T) = {
    CompletableFuture.runAsync(() => {
      val headers = new LinkedMultiValueMap[String, String]()
      if (!server.noAuthentication) {
        headers.add(server.tokenHeader, server.tokenValue)
      }
      headers.add(HttpHeaders.CONTENT_TYPE, ContentType)
      webhookServiceRestTemplate.exchange(
        server.webhookServerURL,
        HttpMethod.POST,
        new HttpEntity[T](requestBody, headers),
        classOf[Void]
      )
    }).whenComplete((_, ex) => {
      Option(ex) match {
        case Some(exception) =>
          logger.warn(s"An error occurred when sending webhook event to ${server.webhookServerURL}: ${exception.getMessage}")
      }
    })
  }
}

final case class DeploymentPackageStatusServerException(message: String) extends DeployitException(message)
final case class DeploymentPackageStatusClientException(message: String) extends DeployitException(message)
